
// VK3PB SOTA BOX program by Peter Berrett VK3PB for the Arduino Uno (freeware);


#include <TinyGPS++.h>
#include <SoftwareSerial.h>  
#include <LiquidCrystal.h>
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);           // select the pins used on the LCD panel
float UTC = 1206;

static const int RXPin = 13, TXPin = 12 ;
static const uint32_t GPSBaud = 9600;

// The TinyGPS++ object
TinyGPSPlus gps;

// The serial connection to the GPS device
SoftwareSerial ss(RXPin, TXPin);


void setup()
{
  //My code was adapted from avrious software examples. I have left in the following
  //lines which send GPS data out the serial port periodically.
  
  Serial.begin(115200);
  ss.begin(GPSBaud);
  lcd.begin(16, 2);               // start the library
  Serial.println(F("DeviceExample.ino"));
  Serial.println(F("A simple demonstration of TinyGPS++ with an attached GPS module"));
  Serial.print(F("Testing TinyGPS++ library v. ")); Serial.println(TinyGPSPlus::libraryVersion());
  Serial.println(F("by Mikal Hart"));
  Serial.println();

}

void loop()
{
  // This sketch displays information every time a new sentence is correctly encoded.
while (ss.available() > 0)
    if (gps.encode(ss.read()))
      displayInfo();
    if (millis() > 5000 && gps.charsProcessed() < 10)
    {
      Serial.println(F("No GPS detected: check wiring."));
    
      while(true);
    }
}

void displayInfo()
{
  Serial.print(F("Location: ")); 
  if (gps.location.isValid())
  {
    Serial.print(gps.location.lat(), 6);
    Serial.print(F(","));
    Serial.print(gps.location.lng(), 6);
  }
  else
  {
    Serial.print(F("INVALID"));
  }

  Serial.print(F("  Date/Time: "));
  if (gps.date.isValid())
  {
    Serial.print(gps.date.month());
    Serial.print(F("/"));
    Serial.print(gps.date.day());
    Serial.print(F("/"));
    Serial.print(gps.date.year());
  }
  else
  {
    Serial.print(F("INVALID"));
  }

  Serial.print(F(" "));
  if (gps.time.isValid())
  {
    if (gps.time.hour() < 10) Serial.print(F("0"));
    Serial.print(gps.time.hour());
    Serial.print(F(":"));
    if (gps.time.minute() < 10) Serial.print(F("0"));
    Serial.print(gps.time.minute());
    Serial.print(F(":"));
    if (gps.time.second() < 10) Serial.print(F("0"));
    Serial.print(gps.time.second());
    Serial.print(F("."));
    if (gps.time.centisecond() < 10) Serial.print(F("0"));
    Serial.print(gps.time.centisecond());
  }
  else
  {
    Serial.print(F("INVALID"));
  }

  Serial.println();

  String UTC="";
  
  if (gps.time.isValid())
  {
    if (gps.time.hour() < 10) UTC=UTC+"0";
    UTC=UTC+(gps.time.hour());
    UTC=UTC+":";
    
    if (gps.time.minute() < 10) UTC=UTC+"0";
    UTC=UTC+(gps.time.minute());
  }
  else
  {
  UTC="Invalid";
  }

  float latitude =(gps.location.lat());
  float longitude = (gps.location.lng());

  
  lcd.clear();
  
  String maidenhead = myConvertFunction (latitude, longitude);
  
  lcd.setCursor(0,0);             // set the LCD cursor   position 
  lcd.print("Lat");  // 
  lcd.setCursor(4,0);             // set the LCD cursor   position 
  lcd.print(latitude,6);  // print latitude
  
  lcd.setCursor(0,1);             // set the LCD cursor   position
  lcd.print("Lon");  // 
  lcd.setCursor(4,1);             // set the LCD cursor   position 
  lcd.print(longitude,6);  // print latitude
  delay(3000);
  lcd.clear();
  
  lcd.setCursor(0,0);             // set the LCD cursor   position
  lcd.print("UTC");  // 
  lcd.setCursor(4,0);             // set the LCD cursor   position 
  lcd.print(UTC);  // print time
  
  lcd.setCursor(0,1);             // set the LCD cursor   position
  lcd.print("MDN");  // 
  lcd.setCursor(4,1);             // set the LCD cursor   position 
  lcd.print(maidenhead);  // print time
  delay(3000);
  lcd.clear();

}

//This funtction works out the Maidenhead Grid Square

String myConvertFunction(float latvar, float longvar){
    
    String maidenhead="";

    // make into positive numbers;
    float latpos= latvar+90;
    float longpos=longvar+180;
    
    //#‎1st‬ letter divide by 20;
    float longfirst=longpos/20;
    char firstlet=char((int(longfirst)+97));
    
    //#‎2nd‬ letter divide by 10;
    float latsecond=latpos/10;
    char secondlet=char((int(latsecond)+97));
    
    // determine remainder;
    float latrem=latpos-(10*int(latsecond));
    float longrem=longpos-(20*int(longfirst));
    
    //#‎3rd‬ letter halve and int;
    char thirdlet=char((int(longrem/2))+48);
    
    //#‎4th‬ letter just int;
    char fourthlet= char((int(latrem)+48));
    
    //determine remainder;
    float latrem2=latrem-(int(latrem));
    float longrem2=longrem-(2*int(longrem/2));
    
    // divide by multiply by 24,12 respectively and take integer;
    int latlet2=int(latrem2*24);
    int longlet2=int(longrem2*12);
    
    //#‎5th‬ letter;
    char fifthlet=char(longlet2+97);
    
    // 6th letter
    char sixthlet=char(latlet2+97);
    
    // add eveything into one string
    maidenhead = maidenhead+firstlet+secondlet+thirdlet+fourthlet+fifthlet+sixthlet;
    return maidenhead;
    
}



